// Copyright (C) 2010 - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function [ X , C , L ] = sudoku_confirmcell ( varargin )
  //   Confirm the value of a cell.
  //
  // Calling Sequence
  //   [ X , C , L ] = sudoku_confirmcell ( X , C , L , i , j , v )
  //   [ X , C , L ] = sudoku_confirmcell ( X , C , L , i , j , v , verbose )
  //
  // Parameters
  // X: a 9-by-9 matrix, with 0 for unknown entries
  // C: a 9 x 9 cell of candidates
  // L: the 9 x 9 matrix of number candidates
  // i: the row of the value
  // j: the column of the value
  // v: the value to remove
  //
  // Description
  //   Confirms v as a value for X(i,j) and remove this value 
  //   from candidates of the row, column and block.
  //
  // Examples
  // X = [
  // 0 2 0   6 3 0   0 4 0
  // 6 0 0   0 0 0   0 0 3
  // 3 0 4   0 0 0   5 6 0
  // ..
  // 0 0 0   8 0 6   0 0 0
  // 8 0 0   0 1 0   0 0 6
  // 0 6 0   7 0 5   0 0 0
  // ..
  // 0 8 7   0 0 0   6 0 4
  // 4 0 0   0 6 7   0 0 8
  // 0 3 6   0 4 8   0 2 0
  // ];
  // [C,L] = sudoku_candidates(X);
  // [ X , C , L ] = sudoku_confirmcell ( X , C , L , 7 , 5 , 5 );
  //
  // Authors
  // Michael Baudin, 2010
  
  [lhs,rhs]=argn();
  if ( rhs < 6 | rhs > 7 ) then
    errmsg = msprintf(gettext("%s: Unexpected number of input arguments : %d provided while 6 or 7 are expected."), "sudoku_confirmcell", rhs);
    error(errmsg)
  end
  X = varargin(1)
  C = varargin(2)
  L = varargin(3)
  i = varargin(4)
  j = varargin(5)
  v = varargin(6)
  if ( rhs < 7 ) then
    verbose = %f
  else
    verbose = varargin(7)
  end

  apifun_typereal ( X , "X" , 1 )
  apifun_typehypermat ( C , "C" , 2 )
  apifun_typereal ( L , "L" , 3 )
  apifun_typereal ( i , "i" , 4 )
  apifun_typereal ( j , "j" , 5 )
  apifun_typereal ( v , "v" , 6 )
  apifun_typeboolean ( verbose , "verbose" , 7 )

  // Confirm v for (i,j)
  X(i,j) = v
  C(i,j,:) = %f
  L(i,j) = 0
  if ( verbose ) then
    mprintf("Confirmed (%d,%d) as %d (remains %d unknowns)\n",i,j,v,sum(X==0))
  end
  // Remove v from row i of C
  C(i,:,v) = %f
  for c = 1 : 9
    L(i,c) = size(find(C(i,c,:)),"*")
  end
  // Remove v from column j of C
  C(:,j,v) = %f
  for r = 1 : 9
    L(r,j) = size(find(C(r,j,:)),"*")
  end
  // Remove v from block (i,j) of C
  C(tri(i),tri(j),v) = %f
  for r = tri(i)
    for c = tri(j)
      L(r,c) = size(find(C(r,c,:)),"*")
    end
  end

endfunction

//
// tri --
//   tri(1) = [1 2 3], tri(2) = [1 2 3], tri(3) = [1 2 3]
//   tri(4) = [4 5 6], tri(5) = [4 5 6], tri(6) = [4 5 6]
//   tri(7) = [7 8 9], tri(8) = [7 8 9], tri(9) = [7 8 9]
function y = tri ( k )
  y = 3*ceil(k/3-1) + (1:3)
endfunction
// Generates an error if the given variable is not of type hypermat
function apifun_typehypermat ( var , varname , ivar )
  if ( typeof ( var ) <> "hypermat" ) then
    errmsg = msprintf(gettext("%s: Expected hypermat variable for variable %s at input #%d, but got %s instead."),"apifun_typehypermat", varname , ivar , typeof(var) );
    error(errmsg);
  end
endfunction

// Generates an error if the given variable is not of type boolean
function apifun_typeboolean ( var , varname , ivar )
  if ( type ( var ) <> 4 ) then
    errmsg = msprintf(gettext("%s: Expected boolean but for variable %s at input #%d, got %s instead."),"apifun_typeboolean", varname , ivar , typeof(var) );
    error(errmsg);
  end
endfunction
// Generates an error if the given variable is not of type real
function apifun_typereal ( var , varname , ivar )
  if ( type ( var ) <> 1 ) then
    errmsg = msprintf(gettext("%s: Expected real variable for variable %s at input #%d, but got %s instead."),"apifun_typereal", varname , ivar , typeof(var) );
    error(errmsg);
  end
endfunction
// Generates an error if the given variable is not of type cell
function apifun_typecell ( var , varname , ivar )
  if ( typeof ( var ) <> "ce" ) then
    errmsg = msprintf(gettext("%s: Expected cell variable for variable %s at input #%d, but got %s instead."),"apifun_typecell", varname , ivar , typeof(var) );
    error(errmsg);
  end
endfunction

